# INTER VEHICLE DISTANCE MODEL
# ----------------------------------------------------------------------------
## Author: Gina Torres
## Date Creation: 14-04-2020
# ----------------------------------------------------------------------------
# The main task of this model is to find the copula that best describes the inter-vehicle distance of vehicles given a determined traffic type.

# - Input
# - WIM Dataset (csv file). 

# - Output
# - #day_SCATTER 
#   - Scatter plots of daily traffic per lane. 
# - #day_HISTOGRAM
#   - histogram plots of daily traffic per lane. 
# - OBS_day#_c_l2 
# - Separate csv files of the observations (input data) divided per lane per day and per traffic type.
# - ECDF_#day_c_l2
# - Plots of the ecdf (empirical distribution function) per lane per day and per traffic type.
# - RHO_C_L2
# - csv file with the correlation coefficient of observations for every day per lane and per traffic type.
# - RHO_SIM_C_L2
# - csv file with the correlation coefficient of the simulated data for every day per lane and per traffic type.
# - MLE_C_L2
# - csv file with the MLE (maximum likelihood estimation) results for every day per lane and per traffic type.
# - FIT_C_L2
# - RData file (list) with the fitting results (values for AIC, BIC and Log Likelihood) for every day per lane and per traffic type
# - FIT_#day_c_l2
# - Comparative results with the fitting results (values for AIC, BIC and Log Likelihood) for every day per lane and per traffic type
# - SIM_COPNUM_C_L2
# - RData file (list) with the simulated values (copula number [0,1]) for every day per lane and per traffic type given its best fitted copula.
# - SIM_REALNUM_C_L2
# - RData file (list) with the simulated values (real units) for every day per lane and per traffic type given its best fitted copula.
# - SCATTER_REAL_#day_c_l2
# - Scatter plots of the observations per lane per day and per traffic type.
# - SCATTER_SIM_#day_c_l2
# - Scatter plots of the simulations per lane per day and per traffic type.
# - SCATTER_MIX_#day_c_l2
# - Scatter plots of the observations VS. simulations per lane per day and per traffic type.
graphics.off()
rm(list = ls())
# ----------------------------------------------------------------------------
## LOADING LIBRARIES
# ----------------------------------------------------------------------------
library(VineCopula) # For copula fitting and simulation
library(ggplot2) #for plotting
library(chron)   #for days and hours function
library(dplyr)  #used for function select
library(reshape2)  #for melt function (for barplots)
library(lubridate)  #for using as_date function
library(invgamma)  #for fitting pdfs
library(rstudioapi) # for being able to set working directory as project directory

# ----------------------------------------------------------------------------
## ADD PATHS/ DIRECTORY
# ----------------------------------------------------------------------------
#Path for current directory
Sys.time()
# Path for results location
path_data    <- "add input path here"
path_results <- "add output path here"
path_source <-"add path of functions"
#Name of the dataset
name_file<-"RW_12L_April2013_NS.csv"
# ----------------------------------------------------------------------------
## LOADING EXTERNAL FUNCTIONS
# ----------------------------------------------------------------------------
# Self-made functions have to be made in RStudio otherwise the program can't read it

fun<-paste(path_source,"COP_FIT.R",sep='')
source(fun)

fun1<-paste(path_source,"SIMULATION.R",sep='')
source(fun1)
# ----------------------------------------------------------------------------
## LOADING DATA
# ----------------------------------------------------------------------------
data<-read.csv(paste(path_data,name_file,sep=''))
#order data chronologically according to the time stamp
data<-data[order(data$TIME),] 

# ----------------------------------------------------------------------------
## CLEANING AND EDITING DATA
# ----------------------------------------------------------------------------
#That number is substracted in order to get the correct starting date in the data set. 

data$TIME_OK<- data$TIME-719527
#converting to date format
data$TIME_DATE<-as_date(data$TIME_OK)

#converting to date format
as_datetime <- function(x) as.POSIXct("1970-01-01") + as.difftime(x, units = "days")  #1970 is the origin for R but for excel is 1900
data$TIME_ALL<-as_datetime(data$TIME_OK)

# Create a column for days, hours and the respective day of the week 
data$month<- month(data$TIME_ALL)
data$day<- day(data$TIME_ALL)  #COLUMN day is an integer
data$hour<- hour(data$TIME_ALL) 
data$DAY_WEEK<-weekdays(data$TIME_ALL)


# Just select data that is needed.
data<-select(data,'TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','month','day','hour','LANE','SPEED','NEWSUBCAT')

data<-select(filter(data, month==4), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))

# ----------------------------------------------------------------------------
## EXTRACTION OF DATA
# ----------------------------------------------------------------------------
#Days of the month to analyze
# Some days were not included since they had just 1 car or not cars at all
day_s<-c(4:8,10:15,17:22,24:29) 

#Defining hours for traffic types
cong_hour<-c(5,6)             #congestion 
f1<-c(as.numeric(0:(cong_hour[1]-1))) #free flow a
f2<-c(as.numeric((cong_hour[-1]+1):23)) #free flow b

#prelocate lists
days<-list() 
l2<-list()         #data for lane 2
l3<-list()         #data for lane 3
c_l2 <-list()      #congestion_lane2
c_l3 <-list()      #congestion_lane3
f_l2_a <-list()    #free flow lane 2 before congestion hour
f_l2_b <-list()    #free flow lane 2 after congestion hour
f_l3_a <-list()    #free flow lane 3 before congestion hour
f_l3_b <-list()    #free flow lane 3 after congestion hour

#vector of zeros to prelocate the names of the days as characters
day_nam<-rep(0,length(day_s)) 
#loop over days
for (i in 1:length(day_s)) 
{
  #Extraction of data per day
  day_nam[i]<-paste('day',day_s[i],sep='')
  days[[day_nam[i]]]<-select(filter(data, day == day_s[i]), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  #print(x<-dim(as.data.frame(days[[day_nam[i]]]))) #activate to see total num data per day
  
  #Extraction of daily data per lane
  l2[[day_nam[i]]]<-select(filter(days[[day_nam[i]]], LANE == 'RW-12-L-2'), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  l3[[day_nam[i]]]<-select(filter(days[[day_nam[i]]], LANE == 'RW-12-L-3'), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  
  #Extraction of daily data per lane at congestion hour
  c_l2[[day_nam[i]]]<-select(filter(l2[[day_nam[i]]], hour == cong_hour[1] | hour == cong_hour[2] ), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  c_l3[[day_nam[i]]]<-select(filter(l3[[day_nam[i]]], hour == cong_hour[1] | hour == cong_hour[2] ), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  
  #Extraction of daily data per lane at free flow
  f_l2_a[[day_nam[i]]]<-select(filter(l2[[day_nam[i]]], hour ==f1[1] | hour ==f1[2] | hour ==f1[3] | hour ==f1[4] | hour ==f1[5]), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  f_l2_b[[day_nam[i]]]<-select(filter(l2[[day_nam[i]]], hour ==f2[1] | hour ==f2[2] | hour ==f2[3] | hour ==f2[4] | hour ==f2[5] | hour ==f2[6] | hour ==f2[7] | hour ==f2[8] | hour ==f2[9]  | hour ==f2[10] | hour ==f2[11] | hour ==f2[12] | hour ==f2[13] | hour ==f2[14] | hour ==f2[15] | hour ==f2[16] | hour ==f2[17]), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  
  f_l3_a[[day_nam[i]]]<-select(filter(l3[[day_nam[i]]], hour ==f1[1] | hour ==f1[2] | hour ==f1[3] | hour ==f1[4] | hour ==f1[5]), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT'))
  f_l3_b[[day_nam[i]]]<-select(filter(l3[[day_nam[i]]], hour ==f2[1] | hour ==f2[2] | hour ==f2[3] | hour ==f2[4] | hour ==f2[5] | hour ==f2[6] | hour ==f2[7] | hour ==f2[8] | hour ==f2[9]  | hour ==f2[10] | hour ==f2[11] | hour ==f2[12] | hour ==f2[13] | hour ==f2[14] | hour ==f2[15] | hour ==f2[16] | hour ==f2[17]), c('TIME_OK','TIME_DATE','TIME_ALL','DAY_WEEK','day','hour','LANE','SPEED','NEWSUBCAT')) 
}


# ----------------------------------------------------------------------------
## VISUALIZATION
# ----------------------------------------------------------------------------
#Plot of scatter plots and histograms

#Loop over days
for (i in 1:length(day_s)) 
{
  ggplot(data=days[[day_nam[i]]],aes(x=TIME_ALL, y=SPEED, color=LANE)) + 
    geom_jitter(size=0.2) + 
    xlim(min(days[[day_nam[i]]]$TIME_ALL),max(days[[day_nam[i]]]$TIME_ALL))+
    ylim(min(data$SPEED)-5,max(data$SPEED)+5) +
    labs(x="Hour", y="Speed [km/h]") + theme(axis.text=element_text(size=14), axis.title.x=element_text(size=16,face="bold"))
  ggsave(path = path_results, filename = paste(day_nam[i],'_SCATTER.png',sep='') ,width=7, height=3) 
  
  ggplot(data=days[[day_nam[i]]],aes(x=TIME_ALL,color=LANE,)) + 
    geom_histogram(binwidth=3600, fill="white", alpha=0.5, position="identity") + 
    labs(x="", y="Frequency") + 
    theme_classic() +
    theme(axis.text=element_text(size=16), axis.title.y=element_text(size=20),
                                      axis.text.x = element_text(angle = 90),
                                      legend.text = element_text(size=16)) +
    xlim(min(days[[day_nam[i]]]$TIME_ALL),max(days[[day_nam[i]]]$TIME_ALL)) +
    ggsave(path = path_results, filename = paste(day_nam[i],'_HISTOGRAM.png',sep=''),width=7, height=5) 
}

#Bin size is 3600 since I would like to have bins of 1 hour and the units are in seconds (3600 seconds)
# ----------------------------------------------------------------------------
## MODEL DEVELOPMENT
# ----------------------------------------------------------------------------
################ FIRST: we organize the data chronologicaly ####################

#loop over days
for (i in 1:length(day_s)) 
{
  c_l2[[day_nam[i]]]<-dplyr::arrange(c_l2[[day_nam[i]]], TIME_ALL) 
  c_l3[[day_nam[i]]]<-dplyr::arrange(c_l3[[day_nam[i]]], TIME_ALL)
  
  f_l2_a[[day_nam[i]]]<-dplyr::arrange(f_l2_a[[day_nam[i]]], TIME_ALL)
  f_l2_b[[day_nam[i]]]<-dplyr::arrange(f_l2_b[[day_nam[i]]], TIME_ALL)
  
  f_l3_a[[day_nam[i]]]<-dplyr::arrange(f_l3_a[[day_nam[i]]], TIME_ALL)
  f_l3_b[[day_nam[i]]]<-dplyr::arrange(f_l3_b[[day_nam[i]]], TIME_ALL)
}

################ SECOND: get dimensions of daily data per traffic type per lane ####################


#prelocate list for dimensions
dim_c_l2 <-list()    #congestion_lane2
dim_c_l3 <-list()    #congestion_lane3
dim_f_l2_a <-list()  #free flow lane 2a
dim_f_l2_b <-list()  #free flow lane 2b
dim_f_l3_a <-list()  #free flow lane 3a
dim_f_l3_b <-list()  #free flow lane 3b


for (i in 1:length(day_s)) 
{
  dim_c_l2[[day_nam[i]]]<-dim((c_l2[[day_nam[i]]]))
  dim_c_l3[[day_nam[i]]]<-dim((c_l3[[day_nam[i]]]))
  
  dim_f_l2_a[[day_nam[i]]]<-dim((f_l2_a[[day_nam[i]]]))
  dim_f_l2_b[[day_nam[i]]]<-dim((f_l2_b[[day_nam[i]]]))
  
  dim_f_l3_a[[day_nam[i]]]<-dim((f_l3_a[[day_nam[i]]]))
  dim_f_l3_b[[day_nam[i]]]<-dim((f_l3_b[[day_nam[i]]]))
}

# dim_c_l2
# dim_c_l3 
# dim_f_l2_a 
# dim_f_l2_b 
# dim_f_l3_a 
# dim_f_l3_b 
################ THIRD: creating new columns with zeros-> Prelocation within the data frame ####################

for (i in 1:length(day_s)) 
{
  c_l2[[day_nam[i]]]$diff_hour<-0.0
  c_l3[[day_nam[i]]]$diff_hour<-0.0
  
  f_l2_a[[day_nam[i]]]$diff_hour<-0.0
  f_l2_b[[day_nam[i]]]$diff_hour<-0.0
  
  f_l3_a[[day_nam[i]]]$diff_hour<-0.0
  f_l3_b[[day_nam[i]]]$diff_hour<-0.0
  
  c_l2[[day_nam[i]]]$dt<-0.0
  c_l3[[day_nam[i]]]$dt<-0.0
  
  f_l2_a[[day_nam[i]]]$dt<-0.0
  f_l2_b[[day_nam[i]]]$dt<-0.0
  
  f_l3_a[[day_nam[i]]]$dt<-0.0
  f_l3_b[[day_nam[i]]]$dt<-0.0
}

################ FOURTH: computing the time differences between passing cars (diff_hour) and computing distances (dt) ####################

#if the gap is big than 10 hours (tolerance) that row will be deleted
tol<-as.numeric(10)

for (i in 1:length(day_s)) 
{
  #diff_hour is in hours (decimals) 
  c_l2[[day_nam[i]]]$diff_hour[2:dim_c_l2[[day_nam[i]]][1]]<-(diff(c_l2[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  c_l3[[day_nam[i]]]$diff_hour[2:dim_c_l3[[day_nam[i]]][1]]<-(diff(c_l3[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  
  f_l2_a[[day_nam[i]]]$diff_hour[2:dim_f_l2_a[[day_nam[i]]][1]]<-(diff(f_l2_a[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  f_l2_b[[day_nam[i]]]$diff_hour[2:dim_f_l2_b[[day_nam[i]]][1]]<-(diff(f_l2_b[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  
  f_l3_a[[day_nam[i]]]$diff_hour[2:dim_f_l3_a[[day_nam[i]]][1]]<-(diff(f_l3_a[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  f_l3_b[[day_nam[i]]]$diff_hour[2:dim_f_l3_b[[day_nam[i]]][1]]<-(diff(f_l3_b[[day_nam[i]]]$TIME_ALL, lag=1)/3600)
  
  #dt is in km
  c_l2[[day_nam[i]]]$dt[2:dim_c_l2[[day_nam[i]]][1]]<-c_l2[[day_nam[i]]]$SPEED[1:(dim_c_l2[[day_nam[i]]][1]-1)]*c_l2[[day_nam[i]]]$diff_hour[2:dim_c_l2[[day_nam[i]]][1]]
  c_l3[[day_nam[i]]]$dt[2:dim_c_l3[[day_nam[i]]][1]]<-c_l3[[day_nam[i]]]$SPEED[1:(dim_c_l3[[day_nam[i]]][1]-1)]*c_l3[[day_nam[i]]]$diff_hour[2:dim_c_l3[[day_nam[i]]][1]]
  
  f_l2_a[[day_nam[i]]]$dt[2:dim_f_l2_a[[day_nam[i]]][1]]<-f_l2_a[[day_nam[i]]]$SPEED[1:(dim_f_l2_a[[day_nam[i]]][1]-1)]*f_l2_a[[day_nam[i]]]$diff_hour[2:dim_f_l2_a[[day_nam[i]]][1]]
  f_l2_b[[day_nam[i]]]$dt[2:dim_f_l2_b[[day_nam[i]]][1]]<-f_l2_b[[day_nam[i]]]$SPEED[1:(dim_f_l2_b[[day_nam[i]]][1]-1)]*f_l2_b[[day_nam[i]]]$diff_hour[2:dim_f_l2_b[[day_nam[i]]][1]]
  
  f_l3_a[[day_nam[i]]]$dt[2:dim_f_l3_a[[day_nam[i]]][1]]<-f_l3_a[[day_nam[i]]]$SPEED[1:(dim_f_l3_a[[day_nam[i]]][1]-1)]*f_l3_a[[day_nam[i]]]$diff_hour[2:dim_f_l3_a[[day_nam[i]]][1]]
  f_l3_b[[day_nam[i]]]$dt[2:dim_f_l3_b[[day_nam[i]]][1]]<-f_l3_b[[day_nam[i]]]$SPEED[1:(dim_f_l3_b[[day_nam[i]]][1]-1)]*f_l3_b[[day_nam[i]]]$diff_hour[2:dim_f_l3_b[[day_nam[i]]][1]]
  
  #SINCE FREE_FLOW A HAS A BIG GAP IN BETWEEN, THAT VALUE HAS TO BE DELETED (TOL)
  f_l2_a[[day_nam[i]]]<-filter(f_l2_a[[day_nam[i]]], diff_hour<tol)
  f_l3_a[[day_nam[i]]]<-filter(f_l3_a[[day_nam[i]]], diff_hour<tol)
}

# NOW CHECK IF THERE IS NEGATIVE NUMBERS (IF SUM IS >0 (MEANS NEGATIVE #S): THEN THE DATA IS NOT ORDERED CHRONOLOGICALLY)

for (i in 1:length(day_s)) 
{
  print(sum(c_l2[[day_nam[i]]]$diff_hour<0))
  print(sum(c_l3[[day_nam[i]]]$diff_hour<0))
  print(sum(f_l2_a[[day_nam[i]]]$diff_hour<0))
  print(sum(f_l2_a[[day_nam[i]]]$diff_hour<0))
  print(sum(f_l3_a[[day_nam[i]]]$diff_hour<0))
  print(sum(f_l3_b[[day_nam[i]]]$diff_hour<0))
}
# ----------------------------------------------------------------------------
## MODEL DEVELOPMENT - VINE-COPULA
# ----------------------------------------------------------------------------
#prelocate data in lists

dt_c_l2 <-list()
dt_c_l3 <-list()
dt_f_l2_a <-list()
dt_f_l2_b <-list()
dt_f_l3_a <-list()
dt_f_l3_b <-list()

for (i in 1:length(day_s)) 
{
  #Creating lists that contain only the dt column (The computed intervehicle distances) of each day (excluding the first value wich is cero). This is to make the code more readable
  a  <- na.omit(c_l2[[day_nam[i]]]$dt[2:(dim_c_l2[[day_nam[i]]][1])])
  a1 <- na.omit(c_l3[[day_nam[i]]]$dt[2:(dim_c_l3[[day_nam[i]]][1])])
  b  <- na.omit(f_l2_a[[day_nam[i]]]$dt[2:(dim_f_l2_a[[day_nam[i]]][1])])
  b1 <- na.omit(f_l3_a[[day_nam[i]]]$dt[2:(dim_f_l3_a[[day_nam[i]]][1])])
  c  <- na.omit(f_l2_b[[day_nam[i]]]$dt[2:(dim_f_l2_b[[day_nam[i]]][1])])
  c1 <- na.omit(f_l3_b[[day_nam[i]]]$dt[2:(dim_f_l3_b[[day_nam[i]]][1])])
  
  #saving the data sets in csv files for future use
  write.table(a, file=paste0(path_results,"OBS",day_nam[i],"_c_l2",".csv"),sep=",",col.names=F, row.names=F)
  write.table(a1,file=paste0(path_results,"OBS",day_nam[i],"_c_l3",".csv"),sep=",",col.names=F, row.names=F)
  write.table(b, file=paste0(path_results,"OBS",day_nam[i],"_f_l2_a",".csv"),sep=",",col.names=F, row.names=F)
  write.table(b1,file=paste0(path_results,"OBS",day_nam[i],"_f_l3_a",".csv"),sep=",",col.names=F, row.names=F)
  write.table(c, file=paste0(path_results,"OBS",day_nam[i],"_f_l2_b",".csv"),sep=",",col.names=F, row.names=F)
  write.table(c1,file=paste0(path_results,"OBS",day_nam[i],"_f_l3_b",".csv"),sep=",",col.names=F, row.names=F)
  
  #Creating ecdf from the observations
  f  <-ecdf(a) 
  f1 <-ecdf(a1) 
  f2 <-ecdf(b) 
  f3 <-ecdf(b1) 
  f4 <-ecdf(c) 
  f5 <-ecdf(c1)
  
  fi<-sort(f(a))  #sorting quantiles ascending
  fi1<-sort(f(a1))
  fi2<-sort(f(b))
  fi3<-sort(f(b1))
  fi4<-sort(f(c))
  fi5<-sort(f(c1))
  
  #converting the vectors into dataframes for plotting
  dat <-data.frame(values=a) 
  dat1<-data.frame(values=a1)
  dat2<-data.frame(values=b)
  dat3<-data.frame(values=b1)
  dat4<-data.frame(values=c)
  dat5<-data.frame(values=c1)
  
  ################################ PLOTTING OF ECDF'S (OBSERVATIONS) ##############################
  ggplot(dat, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_c_l2",".png"),width=8, height=5)
  
  ggplot(dat1, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_c_l3",".png"),width=8, height=5)
  
  ggplot(dat2, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_f_l2_a",".png"),width=8, height=5)
  
  ggplot(dat3, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_f_l3_a",".png"),width=8, height=5)
  
  ggplot(dat4, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_f_l2_b",".png"),width=8, height=5)
  
  ggplot(dat5, aes(values)) + 
    stat_ecdf(geom = "step", color='red') +
    labs(x=bquote(d[t]), y="F(x)") +
    ggsave(path = path_results, filename = paste0("ECDF_",day_nam[i],"_f_l3_b",".png"),width=8, height=5)
  
  #Creating pseudosamples corresponfing to the dt column (computed inter vehicle distances)
  #here:  number of rows is not added 1 because the dimension is already +1 larger
  dt_c_l2[[day_nam[i]]]<- rank(a,ties.method ="average")/(dim_c_l2[[day_nam[i]]][1])
  dt_c_l3[[day_nam[i]]]<- rank(a1,ties.method ="average")/(dim_c_l3[[day_nam[i]]][1])
  
  dt_f_l2_a[[day_nam[i]]]<- rank(b,ties.method ="average")/(dim_f_l2_a[[day_nam[i]]][1])
  dt_f_l2_b[[day_nam[i]]]<- rank(c,ties.method ="average")/(dim_f_l2_b[[day_nam[i]]][1])
  
  dt_f_l3_a[[day_nam[i]]]<- rank(b1,ties.method ="average")/(dim_f_l3_a[[day_nam[i]]][1])
  dt_f_l3_b[[day_nam[i]]]<- rank(c1,ties.method ="average")/(dim_f_l3_b[[day_nam[i]]][1])
}
################################## APPLYING THE MODEL ############################################

#Prelocating lists

#Here: The result from the copula fitting (only the best fit) (Contains MLE and FIT which later are put into separate files)
result_c_l2<-list()
result_c_l3<-list()
result_f_l2_a<-list()
result_f_l2_b<-list()
result_f_l3_a<-list()
result_f_l3_b<-list()

# Here: Correlation of the observations
corr_c_l2<-list()
corr_c_l3<-list()
corr_f_l2_a<-list()
corr_f_l2_b<-list()
corr_f_l3_a<-list()
corr_f_l3_b<-list()

# Here: Results of the MLE (Copula num, parameters.etc) and FIT (values of BIC,AIC and LogLik)
fitsum_c_l2  <-list()
MLE_c_l2 <-list()
fitsum_c_l3  <-list()
MLE_c_l3     <-list()

fitsum_f_l2_a  <-list()
MLE_f_l2_a     <-list()
fitsum_f_l3_a  <-list()
MLE_f_l3_a     <-list()

fitsum_f_l2_b  <-list()
MLE_f_l2_b     <-list()
fitsum_f_l3_b  <-list()
MLE_f_l3_b     <-list()

for (i in 1:length(day_s)) 
{
  #Computing correlations
  corr_c_l2[[day_nam[i]]] <- cor(dt_c_l2[[day_nam[i]]][1:(length(dt_c_l2[[day_nam[i]]])-1)],dt_c_l2[[day_nam[i]]][2:length(dt_c_l2[[day_nam[i]]])],method ="spearman")
  corr_c_l3[[day_nam[i]]] <- cor(dt_c_l3[[day_nam[i]]][1:(length(dt_c_l3[[day_nam[i]]])-1)],dt_c_l3[[day_nam[i]]][2:length(dt_c_l3[[day_nam[i]]])],method ="spearman")
  
  corr_f_l2_a[[day_nam[i]]] <- cor(dt_f_l2_a[[day_nam[i]]][1:(length(dt_f_l2_a[[day_nam[i]]])-1)],dt_f_l2_a[[day_nam[i]]][2:length(dt_f_l2_a[[day_nam[i]]])],method ="spearman")
  corr_f_l2_b[[day_nam[i]]] <- cor(dt_f_l2_b[[day_nam[i]]][1:(length(dt_f_l2_b[[day_nam[i]]])-1)],dt_f_l2_b[[day_nam[i]]][2:length(dt_f_l2_b[[day_nam[i]]])],method ="spearman")
  
  corr_f_l3_a[[day_nam[i]]] <- cor(dt_f_l3_a[[day_nam[i]]][1:(length(dt_f_l3_a[[day_nam[i]]])-1)],dt_f_l3_a[[day_nam[i]]][2:length(dt_f_l3_a[[day_nam[i]]])],method ="spearman")
  corr_f_l3_b[[day_nam[i]]] <- cor(dt_f_l3_b[[day_nam[i]]][1:(length(dt_f_l3_b[[day_nam[i]]])-1)],dt_f_l3_b[[day_nam[i]]][2:length(dt_f_l3_b[[day_nam[i]]])],method ="spearman")
  
  #Aplying the copula function for best fit:
  result_c_l2[[day_nam[i]]] <- COP_FIT(dt_c_l2[[day_nam[i]]][1:(length(dt_c_l2[[day_nam[i]]])-1)],dt_c_l2[[day_nam[i]]][2:length(dt_c_l2[[day_nam[i]]])],10)
  result_c_l3[[day_nam[i]]] <- COP_FIT(dt_c_l3[[day_nam[i]]][1:(length(dt_c_l3[[day_nam[i]]])-1)],dt_c_l3[[day_nam[i]]][2:length(dt_c_l3[[day_nam[i]]])],10)
  
  result_f_l2_a[[day_nam[i]]] <- COP_FIT(dt_f_l2_a[[day_nam[i]]][1:(length(dt_f_l2_a[[day_nam[i]]])-1)],dt_f_l2_a[[day_nam[i]]][2:length(dt_f_l2_a[[day_nam[i]]])],10)
  result_f_l2_b[[day_nam[i]]] <- COP_FIT(dt_f_l2_b[[day_nam[i]]][1:(length(dt_f_l2_b[[day_nam[i]]])-1)],dt_f_l2_b[[day_nam[i]]][2:length(dt_f_l2_b[[day_nam[i]]])],10)
  
  result_f_l3_a[[day_nam[i]]] <- COP_FIT(dt_f_l3_a[[day_nam[i]]][1:(length(dt_f_l3_a[[day_nam[i]]])-1)],dt_f_l3_a[[day_nam[i]]][2:length(dt_f_l3_a[[day_nam[i]]])],10)
  result_f_l3_b[[day_nam[i]]] <- COP_FIT(dt_f_l3_b[[day_nam[i]]][1:(length(dt_f_l3_b[[day_nam[i]]])-1)],dt_f_l3_b[[day_nam[i]]][2:length(dt_f_l3_b[[day_nam[i]]])],10)
  
  #Separating the results lists into FIT and MLE independent files
  fitsum_c_l2[[day_nam[i]]] <-result_c_l2[[day_nam[i]]][1]
  MLE_c_l2[[day_nam[i]]]    <-result_c_l2[[day_nam[i]]][2]
  
  fitsum_c_l3[[day_nam[i]]] <-result_c_l3[[day_nam[i]]][1]
  MLE_c_l3[[day_nam[i]]]    <-result_c_l3[[day_nam[i]]][2]
  
  fitsum_f_l2_a[[day_nam[i]]] <-result_f_l2_a[[day_nam[i]]][1]
  MLE_f_l2_a[[day_nam[i]]]    <-result_f_l2_a[[day_nam[i]]][2]
  
  fitsum_f_l2_b[[day_nam[i]]] <-result_f_l2_b[[day_nam[i]]][1]
  MLE_f_l2_b[[day_nam[i]]]    <-result_f_l2_b[[day_nam[i]]][2]
  
  fitsum_f_l3_a[[day_nam[i]]] <-result_f_l3_a[[day_nam[i]]][1]
  MLE_f_l3_a[[day_nam[i]]]    <-result_f_l3_a[[day_nam[i]]][2]
  
  fitsum_f_l3_b[[day_nam[i]]] <-result_f_l3_b[[day_nam[i]]][1]
  MLE_f_l3_b[[day_nam[i]]]    <-result_f_l3_b[[day_nam[i]]][2]
}
# ----------------------------------------------------------------------------
## SAVING RESULTS
# ----------------------------------------------------------------------------
#Unlisting lists into organized arrays

# The matrix:COLUMNS: RHO values , ROWS: DAYS
rho_c_l2<- matrix(unlist(corr_c_l2), ncol =1, byrow = F)
rho_c_l3<- matrix(unlist(corr_c_l3), ncol =1, byrow = F)

rho_f_l2_a<- matrix(unlist(corr_f_l2_a), ncol =1, byrow = F)
rho_f_l3_a<- matrix(unlist(corr_f_l3_a), ncol =1, byrow = F)

rho_f_l2_b<- matrix(unlist(corr_f_l2_b), ncol =1, byrow = F)
rho_f_l3_b<- matrix(unlist(corr_f_l3_b), ncol =1, byrow = F)

#The matrix: COLUMNS: DAYS. ROWS: RESULTS OF THE MLE PROCESS
mle_c_l2<- matrix(unlist(MLE_c_l2), ncol =length(day_s), byrow = F)
mle_c_l3<- matrix(unlist(MLE_c_l3), ncol =length(day_s), byrow = F)

mle_f_l2_a<- matrix(unlist(MLE_f_l2_a), ncol =length(day_s), byrow = F)
mle_f_l3_a<- matrix(unlist(MLE_f_l3_a), ncol =length(day_s), byrow = F)

mle_f_l2_b<- matrix(unlist(MLE_f_l2_b), ncol =length(day_s), byrow = F)
mle_f_l3_b<- matrix(unlist(MLE_f_l3_b), ncol =length(day_s), byrow = F)

# SAVING THE RESULTS AS CSV FILES INTO THE DESIRED DIRECTORY
write.table(rho_c_l2,file=paste0(path_results,"RHO_C_L2",".csv"),sep=";",col.names=F, row.names=F)
write.table(rho_c_l3,file=paste0(path_results,"RHO_C_L3",".csv"),sep=";",col.names=F, row.names=F)

write.table(rho_f_l2_a,file=paste0(path_results,"RHO_F_L2_A",".csv"),sep=";",col.names=F, row.names=F)
write.table(rho_f_l3_a,file=paste0(path_results,"RHO_F_L3_A",".csv"),sep=";",col.names=F, row.names=F)

write.table(rho_f_l2_b,file=paste0(path_results,"RHO_F_L2_B",".csv"),sep=";",col.names=F, row.names=F)
write.table(rho_f_l3_b,file=paste0(path_results,"RHO_F_L3_B",".csv"),sep=";",col.names=F, row.names=F)

write.table(mle_c_l2,file=paste0(path_results,"MLE_C_L2",".csv"),sep=";",col.names=F, row.names=F)
write.table(mle_c_l3,file=paste0(path_results,"MLE_C_L3",".csv"),sep=";",col.names=F, row.names=F)

write.table(mle_f_l2_a,file=paste0(path_results,"MLE_F_L2_A",".csv"),sep=";",col.names=F, row.names=F)
write.table(mle_f_l3_a,file=paste0(path_results,"MLE_F_L3_A",".csv"),sep=";",col.names=F, row.names=F)

write.table(mle_f_l2_b,file=paste0(path_results,"MLE_F_L2_B",".csv"),sep=";",col.names=F, row.names=F)
write.table(mle_f_l3_b,file=paste0(path_results,"MLE_F_L3_B",".csv"),sep=";",col.names=F, row.names=F)

# SAVING THE RESULTS AS RDATA FILES (LISTS) INTO THE DESIRED DIRECTORY
save(fitsum_c_l2, file=paste(path_results,"FIT_C_L2.RData"))
save(fitsum_c_l3, file=paste(path_results,"FIT_C_L3.RData"))

save(fitsum_f_l2_a, file=paste(path_results,"FIT_F_L2_A.RData"))
save(fitsum_f_l3_a, file=paste(path_results,"FIT_F_L3_A.RData"))

save(fitsum_f_l2_b, file=paste(path_results,"FIT_F_L2_B.RData"))
save(fitsum_f_l3_b, file=paste(path_results,"FIT_F_L3_B.RData"))

############################# Plotting FIT SUMMARY results- Grouped Bar Plot ####################################

for (i in 1:length(day_s)) 
{
  #Lists are converted to dataframes to be able to plot the results.
  # x is a variable that's not saved. It's used just for plotting
  
  # c_l2
  x<-as.data.frame(fitsum_c_l2[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_c_l2",".png"),width=8, height=5) 
  
  #c_l3
  x<-as.data.frame(fitsum_c_l3[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_c_l3",".png"),width=8, height=5) 
  
  #f_l2_a
  x<-as.data.frame(fitsum_f_l2_a[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_f_l2_a",".png"),width=8, height=5) 
  
  #f_l2_b
  x<-as.data.frame(fitsum_f_l2_b[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_f_l2_b",".png"),width=8, height=5) 
  
  #f_l3_a
  x<-as.data.frame(fitsum_f_l3_a[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_f_l3_a",".png"),width=8, height=5) 
  
  #f_l3_b
  x<-as.data.frame(fitsum_f_l3_b[[day_nam[i]]])
  x<- melt(x, id.vars="family", measure.vars=c("LogLik","AIC","BIC"))
  
  ggplot(x, aes(fill=variable, y=value, x=as.character(family))) + 
    geom_bar(position="dodge", stat="identity") +
    labs(x="Copula Family", y="", title="Fitting Results", fill = "") +
    ggsave(path = path_results, filename = paste0("FIT_",day_nam[i],"_f_l3_b",".png"),width=8, height=5) 
}
## SIMULATION
# This simulation is performed just for analysis purposes, the number of simulations = # of observations
# That's why the dimensions of the observations are used as inputs for the SIMULATION function 

# prelocate lists for number of elements per traffic type
N_c_l2  <-list()
N_c_l3  <-list()
N_f_l2_a<-list()
N_f_l2_b<-list()
N_f_l3_a<-list()
N_f_l3_b<-list()

for (i in 1:length(day_s)) 
{
  N_c_l2[[day_nam[i]]]<-length(dt_c_l2[[day_nam[i]]])
  N_c_l3[[day_nam[i]]]<-length(dt_c_l3[[day_nam[i]]])
  
  N_f_l2_a[[day_nam[i]]]<-length(dt_f_l2_a[[day_nam[i]]])
  N_f_l2_b[[day_nam[i]]]<-length(dt_f_l2_b[[day_nam[i]]])
  
  N_f_l3_a[[day_nam[i]]]<-length(dt_f_l3_a[[day_nam[i]]])
  N_f_l3_b[[day_nam[i]]]<-length(dt_f_l3_b[[day_nam[i]]])
}

# N_c_l2  
# N_c_l3  
# N_f_l2_a
# N_f_l2_b
# N_f_l3_a
# N_f_l3_b
#prelocate lists for simulations
sim_c_l2<-list()
sim_c_l3<-list()

sim_f_l2_a<-list()
sim_f_l2_b<-list()
sim_f_l3_a<-list()
sim_f_l3_b<-list()

# Extraction of the MLE results to get the copula number and corresponding parameters
for (i in 1:length(day_s)) 
{
  
  MLE<-as.data.frame(MLE_c_l2[[day_nam[i]]])
  names(MLE)[names(MLE) == colnames(MLE,1)] <- "Parameters"
  MLE[,1]<-as.character(MLE[,1])
  
  MLE1<-as.data.frame(MLE_c_l3[[day_nam[i]]])
  names(MLE1)[names(MLE1) == colnames(MLE1,1)] <- "Parameters"
  MLE1[,1]<-as.character(MLE1[,1])
  
  MLE2<-as.data.frame(MLE_f_l2_a[[day_nam[i]]])
  names(MLE2)[names(MLE2) == colnames(MLE2,1)] <- "Parameters"
  MLE2[,1]<-as.character(MLE2[,1])
  
  MLE3<-as.data.frame(MLE_f_l3_a[[day_nam[i]]])
  names(MLE3)[names(MLE3) == colnames(MLE3,1)] <- "Parameters"
  MLE3[,1]<-as.character(MLE3[,1])
  
  MLE4<-as.data.frame(MLE_f_l2_b[[day_nam[i]]])
  names(MLE4)[names(MLE4) == colnames(MLE4,1)] <- "Parameters"
  MLE4[,1]<-as.character(MLE4[,1])
  
  MLE5<-as.data.frame(MLE_f_l3_b[[day_nam[i]]])
  names(MLE5)[names(MLE5) == colnames(MLE5,1)] <- "Parameters"
  MLE5[,1]<-as.character(MLE5[,1])
  
  # PRINTING OF RESULTS
  print(paste("The best fit for",day_nam[i], "c_l2", MLE[5,1],"PAR1: ",MLE[2,1],"PAR2: ",MLE[3,1]))
  print(paste("The best fit for",day_nam[i], "c_l3", MLE1[5,1],"PAR1: ",MLE1[2,1],"PAR2: ",MLE1[3,1]))
  
  print(paste("The best fit for",day_nam[i], "f_l2_a", MLE2[5,1],"PAR1: ",MLE2[2,1],"PAR2: ",MLE2[3,1]))
  print(paste("The best fit for",day_nam[i], "f_l3_a", MLE3[5,1],"PAR1: ",MLE3[2,1],"PAR2: ",MLE3[3,1]))
  
  print(paste("The best fit for",day_nam[i], "f_l2_b", MLE4[5,1],"PAR1: ",MLE4[2,1],"PAR2: ",MLE4[3,1]))
  print(paste("The best fit for",day_nam[i], "f_l3_b", MLE5[5,1],"PAR1: ",MLE5[2,1],"PAR2: ",MLE5[3,1]))
  
  #Convert MLE results (character) as numeric
  cop_c_l2<-as.numeric(MLE[1,1])
  par_c_l2<-as.numeric(c(MLE[2,1],MLE[3,1]))
  
  cop_c_l3<-as.numeric(MLE1[1,1])
  par_c_l3<-as.numeric(c(MLE1[2,1],MLE1[3,1]))
  
  cop_f_l2_a<-as.numeric(MLE2[1,1])
  par_f_l2_a<-as.numeric(c(MLE2[2,1],MLE2[3,1]))
  
  cop_f_l3_a<-as.numeric(MLE3[1,1])
  par_f_l3_a<-as.numeric(c(MLE3[2,1],MLE3[3,1]))
  
  cop_f_l2_b<-as.numeric(MLE4[1,1])
  par_f_l2_b<-as.numeric(c(MLE4[2,1],MLE4[3,1]))
  
  cop_f_l3_b<-as.numeric(MLE5[1,1])
  par_f_l3_b<-as.numeric(c(MLE5[2,1],MLE5[3,1]))
  
  ############# SIMULATION ##############################
  sim_c_l2[[day_nam[i]]]<- SIMULATION(cop_c_l2,par_c_l2,10000)
  sim_c_l3[[day_nam[i]]]<- SIMULATION(cop_c_l3,par_c_l3,10000)
  
  sim_f_l2_a[[day_nam[i]]]<- SIMULATION(cop_f_l2_a,par_f_l2_a,10000)
  sim_f_l2_b[[day_nam[i]]]<- SIMULATION(cop_f_l2_b,par_f_l2_b,10000)
  
  sim_f_l3_a[[day_nam[i]]]<- SIMULATION(cop_f_l3_a,par_f_l3_a,10000)
  sim_f_l3_b[[day_nam[i]]]<- SIMULATION(cop_f_l3_b,par_f_l3_b,10000)
  
}

# Saving simulated values as copula numers [0,1] (LIST)

save(sim_c_l2, file=paste(path_results,"SIM_COPNUM_C_L2.RData"))
save(sim_c_l3, file=paste(path_results,"SIM_COPNUM_C_L3.RData"))

save(sim_f_l2_a, file=paste(path_results,"SIM_COPNUM_F_L2_A.RData"))
save(sim_f_l3_a, file=paste(path_results,"SIM_COPNUM_F_L3_A.RData"))

save(sim_f_l2_b, file=paste(path_results,"SIM_COPNUM_F_L2_B.RData"))
save(sim_f_l3_b, file=paste(path_results,"SIM_COPNUM_F_L3_B.RData"))

##### CONVERSION TO REAL UNITS ############

# prelocate for simulated data in real units
sim_r_c_l2<-list() 
sim_r_c_l3<-list()
sim_r_f_l2_a<-list()
sim_r_f_l2_b<-list()
sim_r_f_l3_a<-list()
sim_r_f_l3_b<-list()

corr_sim_c_l2<-list()
corr_sim_c_l3<-list()

corr_sim_f_l2_a<-list()
corr_sim_f_l2_b<-list()
corr_sim_f_l3_a<-list()
corr_sim_f_l3_b<-list()

#list for plots
g_all_c_l2<-list()
g_all_c_l3<-list()
g_all_f_l2_a<-list()
g_all_f_l2_b<-list()
g_all_f_l3_a<-list()
g_all_f_l3_b<-list()

for (i in 1:length(day_s)) 
{
  
  #Creating lists that contain only the dt column (The computed intervehicle distances) of each day (excluding the first value wich is cero). This is to make the code more readable
  a  <- c_l2[[day_nam[i]]]$dt[2:(dim_c_l2[[day_nam[i]]][1])]
  a1 <- c_l3[[day_nam[i]]]$dt[2:(dim_c_l3[[day_nam[i]]][1])]
  b  <- f_l2_a[[day_nam[i]]]$dt[2:length(f_l2_a[[day_nam[i]]]$dt)]
  b1 <- f_l3_a[[day_nam[i]]]$dt[2:length(f_l3_a[[day_nam[i]]]$dt)]
  c  <- f_l2_b[[day_nam[i]]]$dt[2:(dim_f_l2_b[[day_nam[i]]][1])]
  c1 <- f_l3_b[[day_nam[i]]]$dt[2:(dim_f_l3_b[[day_nam[i]]][1])]
  
  # units of the results in km
  sim_r_c_l2[[day_nam[i]]]<-as.data.frame(quantile(a,sim_c_l2[[day_nam[i]]]))[,1]
  sim_r_c_l3[[day_nam[i]]]<-as.data.frame(quantile(a1,sim_c_l3[[day_nam[i]]]))[,1]
  sim_r_f_l2_a[[day_nam[i]]]<-as.data.frame(quantile(b,sim_f_l2_a[[day_nam[i]]]))[,1]
  sim_r_f_l3_a[[day_nam[i]]]<-as.data.frame(quantile(b1,sim_f_l3_a[[day_nam[i]]]))[,1]
  sim_r_f_l2_b[[day_nam[i]]]<-as.data.frame(quantile(c,sim_f_l2_b[[day_nam[i]]]))[,1]
  sim_r_f_l3_b[[day_nam[i]]]<-as.data.frame(quantile(c1,sim_f_l3_b[[day_nam[i]]]))[,1]
  
  # computing correlation of simulated data
  corr_sim_c_l2[[day_nam[i]]]<-cor(sim_r_c_l2[[day_nam[i]]][1:(length(sim_r_c_l2[[day_nam[i]]])-1)],sim_r_c_l2[[day_nam[i]]][2:length(sim_r_c_l2[[day_nam[i]]])],method ="spearman")
  corr_sim_c_l3[[day_nam[i]]]<-cor(sim_r_c_l3[[day_nam[i]]][1:(length(sim_r_c_l3[[day_nam[i]]])-1)],sim_r_c_l3[[day_nam[i]]][2:length(sim_r_c_l3[[day_nam[i]]])],method ="spearman")
  corr_sim_f_l2_a[[day_nam[i]]]<-cor(sim_r_f_l2_a[[day_nam[i]]][1:(length(sim_r_f_l2_a[[day_nam[i]]])-1)],sim_r_f_l2_a[[day_nam[i]]][2:length(sim_r_f_l2_a[[day_nam[i]]])],method ="spearman")
  corr_sim_f_l2_b[[day_nam[i]]]<-cor(sim_r_f_l2_b[[day_nam[i]]][1:(length(sim_r_f_l2_b[[day_nam[i]]])-1)],sim_r_f_l2_b[[day_nam[i]]][2:length(sim_r_f_l2_b[[day_nam[i]]])],method ="spearman")
  corr_sim_f_l3_a[[day_nam[i]]]<-cor(sim_r_f_l3_a[[day_nam[i]]][1:(length(sim_r_f_l3_a[[day_nam[i]]])-1)],sim_r_f_l3_a[[day_nam[i]]][2:length(sim_r_f_l3_a[[day_nam[i]]])],method ="spearman")
  corr_sim_f_l3_b[[day_nam[i]]]<-cor(sim_r_f_l3_b[[day_nam[i]]][1:(length(sim_r_f_l3_b[[day_nam[i]]])-1)],sim_r_f_l3_b[[day_nam[i]]][2:length(sim_r_f_l3_b[[day_nam[i]]])],method ="spearman")
  
  # scatter plots for observated data
  ggplot() + aes(x=a[1:(length(a)-1)],y=a[2:length(a)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_c_l2",".png"),width=8, height=5)
  
  ggplot() + aes(x=a1[1:(length(a1)-1)],y=a1[2:length(a1)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_c_l3",".png"),width=8, height=5)
  
  ggplot() + aes(x=b[1:(length(b)-1)],y=b[2:length(b)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_f_l2_a",".png"),width=8, height=5)
  
  ggplot() + aes(x=b1[1:(length(b1)-1)],y=b1[2:length(b1)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_f_l3_a",".png"),width=8, height=5)
  
  ggplot() + aes(x=c[1:(length(c)-1)],y=c[2:length(c)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_f_l2_b",".png"),width=8, height=5)
  
  ggplot() + aes(x=c1[1:(length(c1)-1)],y=c1[2:length(c1)]) +
    geom_point(color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_REAL_",day_nam[i],"_f_l3_b",".png"),width=8, height=5)
  
  # scatter plots for simulated data
  g_all_c_l2[[day_nam[i]]]<- ggplot() + aes(x=sim_r_c_l2[[day_nam[i]]][1:(length(sim_r_c_l2[[day_nam[i]]])-1)],y=sim_r_c_l2[[day_nam[i]]][2:length(sim_r_c_l2[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_c_l2",".png"),width=8, height=5)
  
  g_all_c_l3[[day_nam[i]]]<-ggplot() + aes(x=sim_r_c_l3[[day_nam[i]]][1:(length(sim_r_c_l3[[day_nam[i]]])-1)],y=sim_r_c_l3[[day_nam[i]]][2:length(sim_r_c_l3[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_c_l3",".png"),width=8, height=5)
  
  g_all_f_l2_a[[day_nam[i]]]<-ggplot() + aes(x=sim_r_f_l2_a[[day_nam[i]]][1:(length(sim_r_f_l2_a[[day_nam[i]]])-1)],y=sim_r_f_l2_a[[day_nam[i]]][2:length(sim_r_f_l2_a[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_f_l2_a",".png"),width=8, height=5)
  
  g_all_f_l2_b[[day_nam[i]]]<-ggplot() + aes(x=sim_r_f_l2_b[[day_nam[i]]][1:(length(sim_r_f_l2_b[[day_nam[i]]])-1)],y=sim_r_f_l2_b[[day_nam[i]]][2:length(sim_r_f_l2_b[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_f_l2_b",".png"),width=8, height=5)
  
  g_all_f_l3_a[[day_nam[i]]]<-ggplot() + aes(x=sim_r_f_l3_a[[day_nam[i]]][1:(length(sim_r_f_l3_a[[day_nam[i]]])-1)],y=sim_r_f_l3_a[[day_nam[i]]][2:length(sim_r_f_l3_a[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_f_l3_a",".png"),width=8, height=5)
  
  g_all_f_l3_b[[day_nam[i]]]<-ggplot() + aes(x=sim_r_f_l3_b[[day_nam[i]]][1:(length(sim_r_f_l3_b[[day_nam[i]]])-1)],y=sim_r_f_l3_b[[day_nam[i]]][2:length(sim_r_f_l3_b[[day_nam[i]]])]) +
    geom_point(color='black') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_SIM_",day_nam[i],"_f_l3_b",".png"),width=8, height=5)
  
  # scatter plots for simulated data AND observed data for comparison purposes
  g_all_c_l2[[day_nam[i]]]<- g_all_c_l2[[day_nam[i]]] + geom_point(aes(x=a[1:(length(a)-1)],y=a[2:length(a)]),color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_c_l2",".png"),width=8, height=5)
  
  g_all_c_l3[[day_nam[i]]]<- g_all_c_l3[[day_nam[i]]]+ geom_point(aes(x=a1[1:(length(a1)-1)],y=a1[2:length(a1)]), color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_c_l3",".png"),width=8, height=5)
  
  g_all_f_l2_a[[day_nam[i]]]<- g_all_f_l2_a[[day_nam[i]]] + geom_point(aes(x=b[1:(length(b)-1)],y=b[2:length(b)]), color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_f_l2_a",".png"),width=8, height=5)
  
  g_all_f_l3_a[[day_nam[i]]]<- g_all_f_l3_a[[day_nam[i]]] + geom_point(aes(x=b1[1:(length(b1)-1)],y=b1[2:length(b1)]), color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_f_l3_a",".png"),width=8, height=5)
  
  g_all_f_l2_b[[day_nam[i]]]<- g_all_f_l2_b[[day_nam[i]]] + geom_point(aes(x=c[1:(length(c)-1)],y=c[2:length(c)]), color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_f_l2_b",".png"),width=8, height=5)
  
  g_all_f_l3_b[[day_nam[i]]]<- g_all_f_l3_b[[day_nam[i]]] + geom_point(aes(x=c1[1:(length(c1)-1)],y=c1[2:length(c1)]), color='red') +
    labs(x=bquote(d[t]), y=bquote(d[t+1])) +
    ggsave(path = path_results, filename = paste0("SCATTER_MIX_",day_nam[i],"_f_l3_b",".png"),width=8, height=5)
}

# saving simulated data in real units (RDATA file type)
save(sim_r_c_l2, file=paste(path_results,"SIM_REALNUM_C_L2.RData"))
save(sim_r_c_l3, file=paste(path_results,"SIM_REALNUM_C_L3.RData"))

save(sim_r_f_l2_a, file=paste(path_results,"SIM_REALNUM_F_L2_A.RData"))
save(sim_r_f_l3_a, file=paste(path_results,"SIM_REALNUM_F_L3_A.RData"))

save(sim_r_f_l2_b, file=paste(path_results,"SIM_REALNUM_F_L2_B.RData"))
save(sim_r_f_l3_b, file=paste(path_results,"SIM_REALNUM_F_L3_B.RData"))

# SAVING CORRELATION OF SIMULATED DATASETS
rho_sim_c_l2<- matrix(unlist(corr_sim_c_l2), ncol =1, byrow = F)
rho_sim_c_l3<- matrix(unlist(corr_sim_c_l3), ncol =1, byrow = F)

rho_sim_f_l2_a<- matrix(unlist(corr_sim_f_l2_a), ncol =1, byrow = F)
rho_sim_f_l3_a<- matrix(unlist(corr_sim_f_l3_a), ncol =1, byrow = F)

rho_sim_f_l2_b<- matrix(unlist(corr_sim_f_l2_b), ncol =1, byrow = F)
rho_sim_f_l3_b<- matrix(unlist(corr_sim_f_l3_b), ncol =1, byrow = F)

write.table(rho_sim_c_l2,file=paste0(path_results,"RHO_SIM_C_L2",".csv"),sep=",",col.names=F, row.names=F)
write.table(rho_sim_c_l3,file=paste0(path_results,"RHO_SIM_C_L3",".csv"),sep=",",col.names=F, row.names=F)

write.table(rho_sim_f_l2_a,file=paste0(path_results,"RHO_SIM_F_L2_A",".csv"),sep=",",col.names=F, row.names=F)
write.table(rho_sim_f_l3_a,file=paste0(path_results,"RHO_SIM_F_L3_A",".csv"),sep=",",col.names=F, row.names=F)

write.table(rho_sim_f_l2_b,file=paste0(path_results,"RHO_SIM_F_L2_B",".csv"),sep=",",col.names=F, row.names=F)
write.table(rho_sim_f_l3_b,file=paste0(path_results,"RHO_SIM_F_L3_B",".csv"),sep=",",col.names=F, row.names=F)

#PRINTING CORRELATION REAL VS RESULTS OF CORRELATION

for (i in 1:length(day_s)) 
{
  print(paste("c_l2_",day_nam[i],"Real: ", corr_c_l2[[day_nam[i]]],"Sim: ", corr_sim_c_l2[[day_nam[i]]]))
  print(paste("c_l3_",day_nam[i],"Real: ", corr_c_l3[[day_nam[i]]],"Sim: ", corr_sim_c_l3[[day_nam[i]]]))
  
  print(paste("f_l2_a_",day_nam[i],"Real: ", corr_f_l2_a[[day_nam[i]]],"Sim: ", corr_sim_f_l2_a[[day_nam[i]]]))
  print(paste("f_l3_a_",day_nam[i],"Real: ", corr_f_l3_a[[day_nam[i]]],"Sim: ", corr_sim_f_l3_a[[day_nam[i]]]))
  
  print(paste("f_l2_b_",day_nam[i],"Real: ", corr_f_l2_b[[day_nam[i]]],"Sim: ", corr_sim_f_l2_b[[day_nam[i]]]))
  print(paste("f_l3_b_",day_nam[i],"Real: ", corr_f_l3_b[[day_nam[i]]],"Sim: ", corr_sim_f_l3_b[[day_nam[i]]]))
}
# ----------------------------------------------------------------------------
Sys.time()
# ----------------------------------------------------------------------------