# VINE-FITTING
# ----------------------------------------------------------------------------
## Author: Gina Torres
## Date Creation : 25-09-2020
# ----------------------------------------------------------------------------
# Script to fit all possible vines to a regular vine of 6 nodes and perform simulations
#
#Input: 
#       RData "data" (Data of maximums per cluster)
#       RData "RV_mat" List with all combinations of proposed Vine
#Output: 
#       .	rv_fit.RData : fitting of the vine matrices
#       .	diagnostics.RData: best vine according to AIC,BIC,LOGLIK
#       .	best.RData: vines with smallest aic
#       .	best_sort.RData: sorted best
#       .	best_sort.xlsx: sorted best in excel
#       .	data_p4_xtr.csv: observations on data point 4 of strait 
#       .	my_RV_fit.xlsx: Fit of all the vine matrices
#       .	packageRV_fit.xlsx: fit by the r package
#       .	best.xlsx: Lowest AIC fit from fitting the matrices
#       .	cor_data.xlsx: correlation from observations
#       .	cor_my_rv.xlsx: correlations by vine selected by user
#       .	cor_rv_pack.xlsx: Correlation by simulations of r package
#       .	data.RData : Saving observations.
#       .	Sim_pack.RData: Simulations from the vine proposed by the r pacakge
#       .	Sim_pack.csv: Simulations from the vine proposed by the r pacakge 
#       .	Sim_myRV.RData: Simulations from the vine proposed by user
#       .	Sim_myRV.csv: Simulations from the vine proposed by user

graphics.off()
rm(list = ls())
#-----------------------------------------------------------------------------------------------
# ADD PATHS/DIRECTORY
#-----------------------------------------------------------------------------------------------
#

#Path for location of data
path_data    <- "add input path here"
path_results <- "add output path here"
path_source  <- "add source path here"

# CHANGE SAVE RESULTS INTO VINE FIT. 

nP   <-4   # gridpoint to choose
nvar <-6   # Number of variables

file_nam<-'data_TS.RData'
file_nam1<-'RV_mat.RData'
file_nam2<-'catalog_n6.RData'

file_nam3<-'idx_t9.RData'
file_nam4<-'idx_t10.RData'
file_nam5<-'idx_t11.RData'
file_nam6<-'idx_t12.RData'
file_nam7<-'idx_t13.RData'
file_nam8<-'idx_t14.RData'
#-----------------------------------------------------------------------------------------------
# LOADING LIBRARIES
#-----------------------------------------------------------------------------------------------

library(ggplot2)                  # package for plotting
library(VineCopula)               # package for bivariate analysis (Copulas)
library(corrplot)                 # package to visualize correlation matrices
library (psych)                   # for pair.panels
library(xlsx)                     # to export files into excel sheets
library(RPushbullet)              # to get sms once run is over
key_push<-'o.qH3JfUk3sPU8vbsOBmDvhIkQPBcZRwon'  # pushbullet key (need to be updated once in a while)

# Self-made function
fun<-paste(path_source,"convert_pseudo.R",sep='')
fun1<-paste(path_source,"unit_sn.R",sep='')
fun2<-paste(path_source,"plot_scatt_corr.R",sep='')
source(fun)
source(fun1)
source(fun2)
#-----------------------------------------------------------------------------------------------
# LOAD THE DATA 
#-----------------------------------------------------------------------------------------------
file1<-paste(path_data,file_nam,sep='')
load(file1) #cc_max data organized per grid point when hsTS is dominant

file2<-paste(path_data2,file_nam1,sep='')
load(file2) #RVine_mat: All combinations RVines for my specific tree

file3<-paste(path_data2,file_nam2,sep='')
load(file3) #catalog of vines for n=6

file4<-paste(path_data2,file_nam3,sep='')
load(file4) # index for T9
file5<-paste(path_data2,file_nam4,sep='')
load(file5) # index for T10
file6<-paste(path_data2,file_nam5,sep='')
load(file6) # index for T11
file7<-paste(path_data2,file_nam6,sep='')
load(file7) # index for T12
file8<-paste(path_data2,file_nam7,sep='')
load(file8) # index for T13
file9<-paste(path_data2,file_nam8,sep='')
load(file9) # index for T14

# Pick one grid point to work from now on  (Point 4)
dat<-data[[nP]]
# Export this data to backup and fitting marginal distributions

ncom<-23040 # Number of possible combinations for RVine of 6 nodes (Reference: Counting Vines-O.M-N)

#-----------------------------------------------------------------------------------------------
# TESTING VINES FROM CATALOG
#-----------------------------------------------------------------------------------------------

data_RV<-dat                                         # Renamed like this to avoid altering the script from original form
cor_data<-cor(data_RV, method = "spearman")          # Correlation matrix

#Transform data to pseudo samples for RVine fit 
data_RV_pseudo<-convert_pseudo(data_RV,dim(data_RV)[1])


start_time <- Sys.time()      #--------------- START: Pushbullet code---------------

# Copula fit for all combinations of a vine of 6 nodes
rv_fit<- lapply(1:ncom, function (i) {
  RVineCopSelect(data_RV_pseudo,familyset = c(1:10), RV_mat[[i]] , selectioncrit = "AIC",
                                                     indeptest = FALSE , level = 0.05 , trunclevel = NA, se =FALSE ,
                                                     rotations = FALSE,method="mle",cores = 4)
  
  
  print(paste0('Fit number ', i))
  }) 



#summary(rv_fit[[1]])
#----------------------------------------------------------------------------
# Saving best results from vine fitting (AIC, BIC, LOGLIK)

best<-unlist(lapply(1:ncom, function (i) {cbind(rv_fit[[i]]$AIC,rv_fit[[i]]$BIC,rv_fit[[i]]$logLik)}))
best<-matrix(best,ncol=3,byrow=T)
colnames(best)<-c('AIC','BIC','logLik')

lab_vine<-c(1:23040)                             # Vine labels
best<-cbind(lab_vine,best)                       # Diagnostics + vine labels
best_sort<-best[order(best[ , 2]), ]             # Sorted diagnostics per AIC

AIC_diagnostic<-best_sort[,2]-best_sort[1,2]     # Create the diagnostic for AIC: min(AIC)-AICi
best_sort<-cbind(best_sort,AIC_diagnostic)
best_sort<-best_sort[order(best_sort[ , 5]), ]

print(paste('The best AIC value is RVine number:',best_sort[1,1]))

select<-best_sort[1,1]                             # Indexing of best fit
# Activate to see the plot:
#dev.new()
#plot(rv_fit[[select]])


end_time <- Sys.time()            #-------------- END: Pushbullet code----------
time<-end_time - start_time
pbPost(type = "note", title = "Programa Listo", body = paste(c('Runing time:',as.character(time),'sec'),collapse='')
       , filetype = "text/plain",apikey = key_push, devices = 'OnePlus 5T')

#-----------------------------------------------------------------------------------------------
# SAVE RESULTS VINE FITTING CATALOG
#-----------------------------------------------------------------------------------------------
save(rv_fit, file=paste0(path_results,"rv_fit.RData"))
save(diagnostics, file=paste0(path_results,"diagnostics.RData"))
save(best, file=paste0(path_results,"best.RData"))
save(best_sort, file=paste0(path_results,"best_sort.RData"))
write.xlsx(best_sort,paste0(path_results,'best_sort.xlsx'))

# Select best RVine according to diagnostics results

my_RV<-rv_fit[[select]]

#goodness of fit
GoF<-RVineGofTest(data_RV_pseudo,my_RV , method = "Breymann", statistic = "KS", B = 50,alpha = 2)
#-----------------------------------------------------------------------------------------------
# TESTTING VINES BY THE VINECOPULA PACKAGE
#-----------------------------------------------------------------------------------------------

rv_pack<- RVineStructureSelect(data_RV_pseudo , familyset =c(1:10), type = 0 ,
                               selectioncrit = "AIC" , indeptest =FALSE, level = 0.05,
                               trunclevel = NA, progress = FALSE,
                               weights = NA, treecrit = "AIC", rotations = TRUE, se= FALSE,
                               method = "mle", cores = 4)
mat_VNpack<-rv_pack$Matrix

#---------------------------
# Look for the vine of vine copula package within the catalog OM.
# Range in i, I found by looking at the photos in (per_prufer_code folder) of all the vines.

for (j in 4993:5067){              # Plot since T2
  # First we create the png file
  png(file=paste0("C:/Users/ginatorresalve/Desktop/CANDIDATES/","find","_T",j,".png"),600,600) 
  
  # Then we plot the figure
  plot(RV_obj[[j]], tree = 2, type = 0,edge.labels = NULL, legend.pos = "bottomleft", interactive = FALSE)
  dev.off()
}

# After looking at the T2 of each of the possible vines. The correct vine is 5004
summary(rv_pack)

dev.new()
plot(rv_pack)
diag_pack<-cbind(rv_pack$AIC,rv_pack$BIC,rv_pack$logLik)

# Merge diagnostics from my selection and package to see which is better
final_diag<-rbind(c(my_RV$AIC,my_RV$BIC,my_RV$logLik),diag_pack)
rownames(final_diag)<-c('My Proposal',"Package")
colnames(final_diag)<-c("AIC",'BIC','Loglik')

print(paste('The best AIC value is:',which.min(final_diag[,1]),',The best BIC value is RVine number:',
            which.min(final_diag[,2]),',The best logLik value is RVine number:',which.min(final_diag[,3])))

#-----------------------------------------------------------------------------------------------
# SAMPLING FROM VINE 
#-----------------------------------------------------------------------------------------------
# My Selection
nsim<-300
sim_my_rv<-RVineSim(nsim,my_RV, U = NULL) # U null because data is already in [0,1]

# By VineCopula package
sim_rv_pack<-RVineSim(nsim ,rv_pack , U = NULL)

# Check correlation from simulations
cor_my_rv   <- cor (sim_my_rv,   method = "spearman")
cor_rv_pack <- cor (sim_rv_pack, method = "spearman")

diff_my_rv    <- abs(cor_my_rv   - cor_data)
diff_rv_pack  <- abs(cor_rv_pack - cor_data)
#-----------------------------------------------------------------------------------------------
# PLOT RESULTS
#-----------------------------------------------------------------------------------------------

# Now with the indexes of each tree We extract the AIC values to make the boxplot
aic_t9 <-as.data.frame(cbind(rep('T9',length(best[idx_t9,1])),best[idx_t9,1]))
aic_t10<-as.data.frame(cbind(rep('T10',length(best[idx_t10,1])),best[idx_t10,1]))
aic_t11<-as.data.frame(cbind(rep('T11',length(best[idx_t11,1])),best[idx_t11,1]))
aic_t12<-as.data.frame(cbind(rep('T12',length(best[idx_t12,1])),best[idx_t12,1]))
aic_t13<-as.data.frame(cbind(rep('T13',length(best[idx_t13,1])),best[idx_t13,1]))
aic_t14<-as.data.frame(cbind(rep('T14',length(best[idx_t14,1])),best[idx_t14,1]))

# Bind the AIC of each tree into one data set. Format 2 columns for ggplot2
AIC_TREE<-rbind(aic_t9,aic_t10,aic_t11,aic_t12,aic_t13,aic_t14)
colnames(AIC_TREE)<-c("Tree","AIC") # Changing name of columns

# Convert column to numeric
apply(AIC_TREE, 2, class)   # To check class of columns
cols.num <- c("AIC")
AIC_TREE[cols.num] <- sapply(AIC_TREE[cols.num],as.numeric)  # To change class of columns
sapply(AIC_TREE, class)     # To check class of columns


# Plot boxplot with ggplot2
# Additionally, construct another data frame that has:
# 1) The selected vine form the catalog (Manual input according to results above)
# 2) The value of the vine form the Vinecopula package within the vine catalog

data_points<-data.frame(x=c('T13','T13'), y = c(best_sort[1,2],best_sort[5723,2]), Vine = c('Proposed',"VineCopula Package"))

g<-ggplot(AIC_TREE, mapping=aes(x=Tree, y=AIC)) + geom_boxplot()  +
  geom_point(data = data_points,
             aes(x=x, y=y,color = Vine), size=2) +
  scale_color_manual(values = c("Proposed" = "green", "VineCopula Package" = "red")) + 
  scale_x_discrete(limits=c("T9","T10","T11","T12","T13","T14"))  +# TO CHANGE POSITION OF ELEMNTS IN LEGEND
  theme_classic() +
  theme(legend.position="none")

ggsave(g,path = path_results, filename=paste0("BOX_PLOT",".eps"),width=7, height=5) 

#--------------------------------------------
# Plot of simulations 

sim1<-sim_rv_pack
sim2<-sim_my_rv

sim_pack<-as.data.frame(sim1)
sim_mine<-as.data.frame(sim2)
obs<-as.data.frame(convert_pseudo(dat,dim(dat)[1]))

# Additional inputs for my own plot function  
break_s<-c(0,1)
limit_s<-c(0,1)
col_label<-c("X^(1)","X^(2)","Y^(1)", "Y^(2)","Z^(1)","Z^(2)")
type<-c("spearman")

ext<-'.eps' 

#Plot data as pseudo obs
g<-plot_scatt_corr(sim_pack,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','SIM_PACK',ext,sep='') ,width=10, height=10)

g<-plot_scatt_corr(sim_mine,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','SIM_MINE',ext,sep='') ,width=10, height=10)

g<-plot_scatt_corr(obs,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','OBS',ext,sep='') ,width=10, height=10)

#plot data as SN
sim_pack_sn<-as.data.frame(qnorm(sim1))
sim_mine_sn<-as.data.frame(qnorm(sim2))
obs_sn<-as.data.frame(qnorm(convert_pseudo(dat,dim(dat)[1])))

g<-plot_scatt_corr(sim_pack_sn,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','SIM_PACK_SN',ext,sep='') ,width=10, height=10)

g<-plot_scatt_corr(sim_mine_sn,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','SIM_MINE_SN',ext,sep='') ,width=10, height=10)

g<-plot_scatt_corr(obs_sn,break_s,limit_s,col_label,type)
ggsave(g,path = path_results, filename = paste('MULTICOR_','OBS_SN',ext,sep='') ,width=10, height=10)


#-----------------------------------------------------------------------------------------------
# SAVE RESULTS
#-----------------------------------------------------------------------------------------------
#For univariate distribution
write.csv(dat,paste(path_results,'data_p4_xtr_ts.csv'))         
write.xlsx(summary(my_RV),paste(path_results,'my_RV_fit_ts.xlsx'))
write.xlsx(summary(rv_pack),paste(path_results,'packageRV_fit_ts.xlsx'))
write.xlsx(best,paste(path_results,'best_ts.xlsx'))
write.xlsx(cor_data,paste(path_results,'cor_data_ts.xlsx'))
write.xlsx(cor_my_rv,paste(path_results,'cor_my_rv_ts.xlsx'))
write.xlsx(cor_rv_pack,paste(path_results,'cor_rv_pack_ts.xlsx'))

#saving simulations
save(dat, file=paste(path_results,"data_ts.RData"))

save(sim1, file=paste(path_results,"Sim_pack_ts.RData"))
write.csv(sim1,paste(path_results,'Sim_pack_ts.csv'))

save(sim2, file=paste(path_results,"Sim_myRV_ts.RData"))
write.csv(sim2,paste(path_results,'Sim_myRV_ts.csv'))
